package com.gyee.alarm.websocket;


import com.gyee.alarm.init.CacheContext;
import com.gyee.alarm.model.auto.ProEconAlarmInfo;
import com.gyee.alarm.model.vo.AlarmTag;
import com.gyee.alarm.model.vo.AlarmTypeValue;
import com.gyee.alarm.model.vo.AlarmVo;
import com.gyee.alarm.model.vo.DeviceTypeValue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import javax.websocket.EncodeException;
import java.io.IOException;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;


/**
 * 功能描述:
 *
 * @description: ws定时推送
 * @Date: 2022/11/30
 */
@Configuration
@EnableScheduling
public class WebsocketSchedule {

    @Autowired
    private WebSocket webSocket;

    // 第一次延迟1秒后执行，之后按fixedRate的规则每5秒执行一次 fixedRateString 与 fixedRate 意思相同，只是使用字符串的形式。唯一不同的是支持占位符
    @Scheduled(initialDelay=1000, fixedRateString = "5")
    public void pushData() throws EncodeException, IOException {
        ConcurrentHashMap<String, WebSocket> webSocketPushMap = WebSocket.webSocketMap;
        ConcurrentHashMap<String, PushParams> webSocketPushParamsMap = WebSocket.webSocketParamsMap;
        if(!webSocketPushMap.isEmpty()) {
            for (String key : webSocketPushMap.keySet()) {
                // 根据ws连接用户ID获取推送参数
                PushParams pushParams = webSocketPushParamsMap.get(key);

                AlarmTag alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test1");
                alarm.setAlarmType(AlarmTypeValue.WT.getCode());
                alarm.setDeviceType(DeviceTypeValue.WT.getCode());
                webSocket.sendMessageByUserId(key, alarm);
                alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test2");
                alarm.setAlarmType(AlarmTypeValue.IN.getCode());
                alarm.setDeviceType(DeviceTypeValue.IN.getCode());
                webSocket.sendMessageByUserId(key, alarm);

                alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test3");
                alarm.setAlarmType(AlarmTypeValue.BT.getCode());
                alarm.setDeviceType(DeviceTypeValue.BT.getCode());
                webSocket.sendMessageByUserId(key, alarm);

                alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test4");
                alarm.setAlarmType(AlarmTypeValue.BTE.getCode());
                alarm.setDeviceType(DeviceTypeValue.BT.getCode());
                webSocket.sendMessageByUserId(key, alarm);

                alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test5");
                alarm.setAlarmType(AlarmTypeValue.CT.getCode());
                alarm.setDeviceType(DeviceTypeValue.IN.getCode());
                webSocket.sendMessageByUserId(key, alarm);

                alarm = new AlarmTag();
                alarm.setTs(new Date().getTime());
                alarm.setDescription("test6");
                alarm.setAlarmType(AlarmTypeValue.CT.getCode());
                alarm.setDeviceType(DeviceTypeValue.WT.getCode());
                webSocket.sendMessageByUserId(key, alarm);
            }

        }
    }
}
