package com.gyee.alarm.service;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.gyee.alarm.init.CacheContext;
import com.gyee.alarm.mapper.auto.AlarmTsMapper;
import com.gyee.alarm.model.auto.*;
import com.gyee.alarm.model.vo.*;
import com.gyee.alarm.service.auto.IAlarmTsService;
import com.gyee.alarm.util.DateUtils;
import com.gyee.alarm.util.SnowflakeGenerator;
import com.gyee.common.model.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


@Service

public class AlarmHistoryService   {

    @Resource
    private AlarmTsMapper alarmTsMapper;
    @Resource
    private IAlarmTsService alarmTsService;

    public  void updateAlarms(List<AlarmTag> ls) {

        if (com.gyee.alarm.util.StringUtils.notEmp(ls) && !ls.isEmpty()) {

            StringBuilder sb=new StringBuilder();
            Map<String, AlarmSimpleVo> map=new HashMap<>();
            for (AlarmTag alarm : ls) {
                sb.append("'").append(alarm.getId()).append("',");

            }

            String ids=sb.substring(0,sb.length()-1);

            List<AlarmSimpleVo> alarmWtls=alarmTsService.selectLastRowByTbname(AlarmSuperTalbeType.WT.getCode(),ids);
            List<AlarmSimpleVo> alarmCtls=alarmTsService.selectLastRowByTbname(AlarmSuperTalbeType.CT.getCode(),ids);
            List<AlarmSimpleVo> alarmBtls=alarmTsService.selectLastRowByTbname(AlarmSuperTalbeType.BT.getCode(),ids);
            if(!alarmWtls.isEmpty())
            {
                for(AlarmSimpleVo vo:alarmWtls)
                {
                    map.put(vo.getTbName(),vo);
                }
            }

            if(!alarmCtls.isEmpty())
            {
                for(AlarmSimpleVo vo:alarmCtls)
                {
                    map.put(vo.getTbName(),vo);
                }
            }

            if(!alarmBtls.isEmpty())
            {
                for(AlarmSimpleVo vo:alarmBtls)
                {
                    map.put(vo.getTbName(),vo);
                }
            }

            for (AlarmTag alarm : ls) {


                if(map.containsKey(alarm.getId()))
                {
                    AlarmSimpleVo tag=map.get(alarm.getId());
                    alarm.setTs(tag.getTs().getTime());

                    Date begin = new Date(alarm.getEndts());
                    Date end = new Date(alarm.getTs());

                    double value= DateUtils.secondsDiff(begin,end);
                    alarm.setTimeLong(value);
                    alarm.setConfirmed(true);
                }
            }

            alarmTsService.insertList(ls);


        }

    }

    @Value("${home_pagep_size}")
    private int home_pagep_size;
    /**
     * 查询报警信息接口
     * @param begin 开始时间
     * @param end 结束时间
     * @param alarmType 报警类型
     * @param stationid 场站
     * @param description 描述
     * @param pageNum 页数
     * @param pageSize 每页多少条
     * @return
     */
    public Map<String,Object> findAlarmlist(String modelId, long begin,long end ,String alarmType,String stationid,String deviceid,String description,String deviceType,String components, int pageNum, int pageSize) {

        Map<String,Object> map=new HashMap<>();

        List<AlarmVo> vos =new ArrayList<>();
        String superTableName= null;
        StringBuilder descriptionSql=new StringBuilder();
        if(StringUtils.notEmp(alarmType) && StringUtils.notEmp(begin) && StringUtils.notEmp(end) )
        {

//            whereSql.append(" where  ts>=").append(begin).append("  and ts<= ").append(end);
//
//            whereSql.append(" and alarmtype=CONCAT('',").append(alarmType).append(",'')");

            if(AlarmTypeValue.BT.getCode().equals(alarmType))
            {
                superTableName= AlarmSuperTalbeType.BT.getCode();

            }else if(AlarmTypeValue.WT.getCode().equals(alarmType))
            {
                superTableName= AlarmSuperTalbeType.WT.getCode();

            }else if(AlarmTypeValue.CT.getCode().equals(alarmType))
            {
                superTableName= AlarmSuperTalbeType.CT.getCode();

            }
        }

//        if(StringUtils.notEmp(stationid)  )
//        {
//            whereSql.append(" and stationid=CONCAT('',").append(stationid).append(",'')");
//
//        }
//        if(StringUtils.notEmp(deviceid)  )
//        {
//            whereSql.append(" and deviceid= CONCAT('',").append(deviceid).append(",'')");
//
//        }
//
        if(StringUtils.notEmp(description)  )
        {
            descriptionSql.append("%").append(description).append("%");
            description=String.valueOf(descriptionSql);
        }
        if(StringUtils.empty(deviceid)  )
        {
            deviceid=null;
        }

        if(StringUtils.empty(modelId)  )
        {
            modelId=null;
        }

        if(StringUtils.empty(description)  )
        {
            description=null;
        }
        if(StringUtils.empty(deviceType)  )
        {
            deviceType=null;
        }

        if(StringUtils.empty(components)  )
        {
            components=null;
        }
        vos = alarmTsMapper.selectByWtLimit(begin,end,modelId,superTableName, alarmType, stationid, deviceid, description,deviceType,components, pageNum*pageSize, (pageNum*pageSize+pageSize));
        map.put("ls",vos);
        List<Integer> total=alarmTsMapper.selectByWtCount(begin,end,modelId,superTableName, alarmType, stationid, deviceid, description,deviceType,components);
        map.put("total",total.size());
        return map;
    }



    public List<AlarmVo> findAlarmByTypelist(String type) {


        List<AlarmVo> vos=new ArrayList<>();


        if(StringUtils.notEmp(type) )
        {
            Date endDate = new Date();
//            Date beginDate = DateUtils.addHours(endDate,-1);
            Date beginDate = DateUtils.truncate(endDate);
            if(type.equals(AlarmTypeValue.BT.getCode()))
            {
                vos = alarmTsMapper.selectByWtLimit(beginDate.getTime(),endDate.getTime(),null,AlarmSuperTalbeType.BT.getCode(), AlarmTypeValue.BT.getCode(), null, null, null,null,null, 1, home_pagep_size);

            }else if(type.equals(AlarmTypeValue.WT.getCode()))
            {
                vos = alarmTsMapper.selectByWtLimit(beginDate.getTime(),endDate.getTime(),null,AlarmSuperTalbeType.WT.getCode(), AlarmTypeValue.WT.getCode(), null, null, null,null, null,1, home_pagep_size);

            }else if(type.equals(AlarmTypeValue.CT.getCode()))
            {
                vos = alarmTsMapper.selectByWtLimit(beginDate.getTime(),endDate.getTime(),null,AlarmSuperTalbeType.CT.getCode(), AlarmTypeValue.CT.getCode(), null, null, null,null, null,1, home_pagep_size);

            }

        }

        return vos;
    }


    public List<AlarmStatVo> findWtFeatureStat(String stationId,String modelId, Date startTime, Date endTime,String components,String alarmIds,String timeType) {
        List<AlarmStatVo> resultList = new ArrayList<>();

        StringBuilder alarmIdList=new StringBuilder();
        StringBuilder componentsList=new StringBuilder();

        Map<String,String> alarmIdMap=new HashMap<>();
        Map<String,String> componentsMap=new HashMap();

        if(StringUtils.notEmp(components))
        {
            String [] componentsStr=components.split(",");
            for(String c:componentsStr)
            {
                componentsList.append("'").append(c).append("',");
                componentsMap.put(c,c);
            }

        }
        if(StringUtils.notEmp(alarmIds))
        {
            String [] alarmIdStr=alarmIds.split(",");
            for(String c:alarmIdStr)
            {
                alarmIdList.append("'").append(c).append("',");
                alarmIdMap.put(c,c);
            }

        }
        String alarmId=null;
        String component=null;
        if(alarmIdList.length()>0)
        {
             alarmId=String.valueOf(alarmIdList.substring(0,alarmIdList.length()-1));
        }
        if(componentsList.length()>0)
        {
            component=String.valueOf(componentsList.substring(0,componentsList.length()-1));
        }
        List<ProBasicFeatureStat> fsls = alarmTsMapper.findWtFeatureStat(startTime.getTime(), endTime.getTime(),stationId,modelId,component,alarmId);


        Map<String, ProBasicFeatureStat> map = new HashMap<>();
        Date statDate=DateUtils.truncate(new Date());

        if (!fsls.isEmpty()) {
            for (ProBasicFeatureStat fs : fsls) {
                if (StringUtils.notEmp(fs.getTypeCode()) && CacheContext.alarmTypeMap.containsKey(fs.getTypeCode())) {
                    ProEconAlarmType at = CacheContext.alarmTypeMap.get(fs.getTypeCode());
                    fs.setTypeName(at.getName());
                }
                fs.setStatDate(new Timestamp(statDate.getTime()));
                fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
                StringBuilder sb = new StringBuilder();
                sb.append(fs.getWtId()).append("_").append(fs.getAlarmid());
                map.put(String.valueOf(sb), fs);
            }
        }
        List<ProBasicFeatureStat> vos = new ArrayList<>();

        if(CacheContext.wpwtmap.containsKey(stationId))
        {
            List<ProBasicEquipment> wtls=CacheContext.wpwtmap.get(stationId);
            for (ProBasicEquipment wt : wtls) {

                StringBuilder sb=new StringBuilder();
                sb.append(wt.getWindpowerstationId()).append(wt.getModelId());
                if(CacheContext.wtAlarmMap.containsKey(String.valueOf(sb))) {
                    List<ProEconAlarmConfiguration> ls = CacheContext.wtAlarmMap.get(String.valueOf(sb));
                    for (ProEconAlarmConfiguration at : ls) {
                        if(StringUtils.notEmp(components) )
                        {
                            if(componentsMap.containsKey(at.getComponents()) )
                            {
                                if(StringUtils.notEmp(alarmIdMap) && !alarmIdMap.isEmpty())
                                {
                                    if( alarmIdMap.containsKey(at.getId()))
                                    {
                                        setWtValues(statDate, map, vos, wt, at);
                                    }

                                }else
                                {
                                    setWtValues(statDate, map, vos, wt, at);
                                }
                            }
                        }else
                        {
                            setWtValues(statDate, map, vos, wt, at);
                        }

                    }
                }
            }
        }


        Map<String,List<AlarmStatVo>> vomap=new HashMap<>();
        for(ProBasicFeatureStat fs:vos)
        {
            AlarmStatVo d = new AlarmStatVo();
            d.setRelateParts(fs.getTypeCode());
            d.setRelatePartsText(fs.getTypeName());
            d.setAlertText(fs.getDescription());
            d.setCount(fs.getTotal());

            if(StringUtils.notEmp(timeType))
            {
                if(AlarmTime.H.getCode().equals(timeType))
                {
                    double times= new BigDecimal(fs.getTotalSeconds()).divide(new BigDecimal(360), 2, RoundingMode.HALF_EVEN).doubleValue();
                    d.setTime(times);
                }else if(AlarmTime.M.getCode().equals(timeType))
                {
                    double times= new BigDecimal(fs.getTotalSeconds()).divide(new BigDecimal(60), 2, RoundingMode.HALF_EVEN).doubleValue();
                    d.setTime(times);
                }else
                {
                    d.setTime(fs.getTotalSeconds());
                }
            }else
            {
                d.setTime(fs.getTotalSeconds());
            }

            d.setStationId(fs.getStationId());
            d.setSnapId(SnowflakeGenerator.generateId());
            d.setWindturbineId(fs.getWtId());
            d.setAlarmid(fs.getAlarmid());
            if(CacheContext.wtmap.containsKey(fs.getWtId()))
            {
                ProBasicEquipment wt= CacheContext.wtmap.get(fs.getWtId());
                d.setWindturbineCode(wt.getNemCode());
            }

            StringBuilder sb=new StringBuilder();
            sb.append(fs.getWtId()).append("_").append(fs.getTypeCode()).append("_").append(fs.getAlarmid());

            if(map.containsKey(String.valueOf(sb)))
            {
                List<AlarmStatVo> ls=vomap.get(String.valueOf(sb));
                ls.add(d);
            }else
            {
                List<AlarmStatVo> ls=new ArrayList<>();
                ls.add(d);
                vomap.put(String.valueOf(sb),ls);
            }
            resultList.add(d);
        }




        return resultList;
    }
    private  static void setWtValues(Date statDate,Map<String, ProBasicFeatureStat> map, List<ProBasicFeatureStat> vos, ProBasicEquipment wt, ProEconAlarmConfiguration at) {

        ProBasicFeatureStat fs = new ProBasicFeatureStat();

        StringBuilder sb = new StringBuilder();
        sb.append(fs.getWtId()).append("_").append(at.getId());
        if (map.containsKey(String.valueOf(sb))) {
            ProBasicFeatureStat old = map.get(String.valueOf(sb));
            BeanUtil.copyProperties(old, fs);
        } else {
            fs.setStatDate(new Timestamp(statDate.getTime()));
            fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
            fs.setWtId(wt.getId());
            fs.setStationId(wt.getWindpowerstationId());
            fs.setTypeCode(at.getComponents());
            if(CacheContext.alarmTypeMap.containsKey(at.getComponents()))
            {
                fs.setTypeName(CacheContext.alarmTypeMap.get(at.getComponents()).getName());
            }
            fs.setAlarmid(at.getId());
            fs.setDescription(at.getDescription());
            fs.setTotal(0l);
            fs.setAvgSeconds(0.0);
            fs.setMaxSeconds(0.0);
            fs.setMinSeconds(0.0);
            fs.setTotalSeconds(0.0);

        }
        vos.add(fs);
    }

    public List<AlarmStatVo> findCtFeatureStat(String stationId,String modelId, Date startTime, Date endTime,String components,String alarmIds,String timeType) {
        List<AlarmStatVo> resultList = new ArrayList<>();


        List<String> alarmIdList=new ArrayList();
        List<String> componentsList=new ArrayList();

        Map<String,String> alarmIdMap=new HashMap<>();
        Map<String,String> componentsMap=new HashMap();

        if(StringUtils.notEmp(components))
        {
            String [] componentsStr=components.split(",");
            for(String c:componentsStr)
            {
                componentsList.add(c);
                componentsMap.put(c,c);
            }

        }
        if(StringUtils.notEmp(alarmIds))
        {
            String [] alarmIdStr=alarmIds.split(",");
            for(String c:alarmIdStr)
            {
                alarmIdList.add(c);
                alarmIdMap.put(c,c);
            }

        }
        List<ProBasicFeatureStat> fsls = alarmTsMapper.findCtFeatureStat(startTime.getTime(), endTime.getTime(),stationId,modelId,componentsList,alarmIdList);


        Map<String, ProBasicFeatureStat> map = new HashMap<>();
        Date statDate=DateUtils.truncate(new Date());

        if (!fsls.isEmpty()) {
            for (ProBasicFeatureStat fs : fsls) {
                if (StringUtils.notEmp(fs.getTypeCode()) && CacheContext.alarmTypeMap.containsKey(fs.getTypeCode())) {
                    ProEconAlarmType at = CacheContext.alarmTypeMap.get(fs.getTypeCode());
                    fs.setTypeName(at.getName());
                }
                fs.setStatDate(new Timestamp(statDate.getTime()));
                fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
                StringBuilder sb = new StringBuilder();
                sb.append(fs.getWtId()).append("_").append(fs.getAlarmid());
                map.put(String.valueOf(sb), fs);
            }
        }
        List<ProBasicFeatureStat> vos = new ArrayList<>();

        if(CacheContext.wpwtmap.containsKey(stationId))
        {
            List<ProBasicEquipment> wtls=CacheContext.wpwtmap.get(stationId);
            for (ProBasicEquipment wt : wtls) {

                StringBuilder sb=new StringBuilder();
                sb.append(wt.getWindpowerstationId()).append(wt.getModelId());

                if(CacheContext.ctAlarmMap.containsKey(String.valueOf(sb))) {

                    List<ProEconAlarmRule> wtCustomls=CacheContext.alarmRulesMap.get(AlarmCustomType.WT.getCode());

                    for (ProEconAlarmRule at : wtCustomls) {

                        if(StringUtils.notEmp(components) )
                        {
                            if(componentsMap.containsKey(at.getRelatedParts()) )
                            {
                                if(StringUtils.notEmp(alarmIdMap) && !alarmIdMap.isEmpty())
                                {
                                    if( alarmIdMap.containsKey(at.getId()))
                                    {
                                        setCtValues(statDate, map, vos, wt, at);
                                    }

                                }else
                                {
                                    setCtValues(statDate, map, vos, wt, at);
                                }
                            }
                        }else
                        {
                            setCtValues(statDate, map, vos, wt, at);
                        }

                    }

                }
            }
        }


        Map<String,List<AlarmStatVo>> vomap=new HashMap<>();
        for(ProBasicFeatureStat fs:vos)
        {
            AlarmStatVo d = new AlarmStatVo();
            d.setRelateParts(fs.getTypeCode());
            d.setRelatePartsText(fs.getTypeName());
            d.setAlertText(fs.getDescription());
            d.setCount(fs.getTotal());
            if(StringUtils.notEmp(timeType))
            {
                if(AlarmTime.H.getCode().equals(timeType))
                {
                    double times= new BigDecimal(fs.getTotalSeconds()).divide(new BigDecimal(360), 2, RoundingMode.HALF_EVEN).doubleValue();
                    d.setTime(times);
                }else if(AlarmTime.M.getCode().equals(timeType))
                {
                    double times= new BigDecimal(fs.getTotalSeconds()).divide(new BigDecimal(60), 2, RoundingMode.HALF_EVEN).doubleValue();
                    d.setTime(times);
                }else
                {
                    d.setTime(fs.getTotalSeconds());
                }
            }else
            {
                d.setTime(fs.getTotalSeconds());
            }
            d.setStationId(fs.getStationId());
            d.setSnapId(SnowflakeGenerator.generateId());
            d.setWindturbineId(fs.getWtId());
            if(CacheContext.wtmap.containsKey(fs.getWtId()))
            {
                ProBasicEquipment wt= CacheContext.wtmap.get(fs.getWtId());
                d.setWindturbineCode(wt.getNemCode());
            }

            StringBuilder sb=new StringBuilder();
            sb.append(fs.getWtId()).append("_").append(fs.getTypeCode()).append("_").append(fs.getAlarmid());

            if(map.containsKey(String.valueOf(sb)))
            {
                List<AlarmStatVo> ls=vomap.get(String.valueOf(sb));
                ls.add(d);
            }else
            {
                List<AlarmStatVo> ls=new ArrayList<>();
                ls.add(d);
                vomap.put(String.valueOf(sb),ls);
            }
            resultList.add(d);
        }


        return resultList;
    }
    private  static void setCtValues(Date statDate,Map<String, ProBasicFeatureStat> map, List<ProBasicFeatureStat> vos, ProBasicEquipment wt, ProEconAlarmRule at) {

        ProBasicFeatureStat fs = new ProBasicFeatureStat();

        StringBuilder sb = new StringBuilder();
        sb.append(fs.getWtId()).append("_").append(at.getId());
        if (map.containsKey(String.valueOf(sb))) {
            ProBasicFeatureStat old = map.get(String.valueOf(sb));
            BeanUtil.copyProperties(old, fs);
        } else {
            fs.setStatDate(new Timestamp(statDate.getTime()));
            fs.setId(String.valueOf(SnowflakeGenerator.generateId()));
            fs.setWtId(wt.getId());
            fs.setStationId(wt.getWindpowerstationId());
            fs.setTypeCode(at.getCategory());
            if(CacheContext.alarmTypeMap.containsKey(at.getCategory()))
            {
                fs.setTypeName(CacheContext.alarmTypeMap.get(at.getCategory()).getName());
            }
            fs.setAlarmid(at.getId());
            fs.setDescription(at.getDescription());
            fs.setTotal(0l);
            fs.setAvgSeconds(0.0);
            fs.setMaxSeconds(0.0);
            fs.setMinSeconds(0.0);
            fs.setTotalSeconds(0.0);

        }
        vos.add(fs);
    }
}
